@@ -1,5 +1,5 @@
 # connectors/mxodbc.py
-# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -21,18 +21,16 @@ For more info on mxODBC, see http://www.egenix.com/
 import sys
 import re
 import warnings
-from decimal import Decimal
 
-from sqlalchemy.connectors import Connector
-from sqlalchemy import types as sqltypes
-import sqlalchemy.processors as processors
+from . import Connector
+
 
 class MxODBCConnector(Connector):
-    driver='mxodbc'
+    driver = 'mxodbc'
 
     supports_sane_multi_rowcount = False
-    supports_unicode_statements = False
-    supports_unicode_binds = False
+    supports_unicode_statements = True
+    supports_unicode_binds = True
 
     supports_native_decimal = True
 
@@ -50,7 +48,7 @@ class MxODBCConnector(Connector):
         elif platform == 'darwin':
             from mx.ODBC import iODBC as module
         else:
-            raise ImportError, "Unrecognized platform for mxODBC import"
+            raise ImportError("Unrecognized platform for mxODBC import")
         return module
 
     @classmethod
@@ -76,8 +74,8 @@ class MxODBCConnector(Connector):
         emit Python standard warnings.
         """
         from mx.ODBC.Error import Warning as MxOdbcWarning
-        def error_handler(connection, cursor, errorclass, errorvalue):
 
+        def error_handler(connection, cursor, errorclass, errorvalue):
             if issubclass(errorclass, MxOdbcWarning):
                 errorclass.__bases__ = (Warning,)
                 warnings.warn(message=str(errorvalue),
@@ -109,9 +107,9 @@ class MxODBCConnector(Connector):
         opts.pop('database', None)
         return (args,), opts
 
-    def is_disconnect(self, e):
-        # eGenix recommends checking connection.closed here,
-        # but how can we get a handle on the current connection?
+    def is_disconnect(self, e, connection, cursor):
+        # TODO: eGenix recommends checking connection.closed here
+        # Does that detect dropped connections ?
         if isinstance(e, self.dbapi.ProgrammingError):
             return "connection already closed" in str(e)
         elif isinstance(e, self.dbapi.Error):
@@ -120,7 +118,7 @@ class MxODBCConnector(Connector):
             return False
 
     def _get_server_version_info(self, connection):
-        # eGenix suggests using conn.dbms_version instead 
+        # eGenix suggests using conn.dbms_version instead
         # of what we're doing here
         dbapi_con = connection.connection
         version = []
@@ -133,21 +131,19 @@ class MxODBCConnector(Connector):
                 version.append(n)
         return tuple(version)
 
-    def do_execute(self, cursor, statement, parameters, context=None):
+    def _get_direct(self, context):
         if context:
             native_odbc_execute = context.execution_options.\
                                         get('native_odbc_execute', 'auto')
-            if native_odbc_execute is True:
-                # user specified native_odbc_execute=True
-                cursor.execute(statement, parameters)
-            elif native_odbc_execute is False:
-                # user specified native_odbc_execute=False
-                cursor.executedirect(statement, parameters)
-            elif context.is_crud:
-                # statement is UPDATE, DELETE, INSERT
-                cursor.execute(statement, parameters)
-            else:
-                # all other statements
-                cursor.executedirect(statement, parameters)
+            # default to direct=True in all cases, is more generally
+            # compatible especially with SQL Server
+            return False if native_odbc_execute is True else True
         else:
-            cursor.executedirect(statement, parameters)
+            return True
+
+    def do_executemany(self, cursor, statement, parameters, context=None):
+        cursor.executemany(
+            statement, parameters, direct=self._get_direct(context))
+
+    def do_execute(self, cursor, statement, parameters, context=None):
+        cursor.execute(statement, parameters, direct=self._get_direct(context))
