@@ -1,11 +1,13 @@
 # informix/base.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 # coding: gbk
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""Support for the Informix database.
+"""
+.. dialect:: informix
+    :name: Informix
 
 .. note::
 
@@ -142,7 +144,9 @@ RESERVED_WORDS = set(
     "xadatasource", "xid", "xload", "xunload", "year"
     ])
 
+
 class InfoDateTime(sqltypes.DateTime):
+
     def bind_processor(self, dialect):
         def process(value):
             if value is not None:
@@ -151,7 +155,9 @@ class InfoDateTime(sqltypes.DateTime):
             return value
         return process
 
+
 class InfoTime(sqltypes.Time):
+
     def bind_processor(self, dialect):
         def process(value):
             if value is not None:
@@ -169,33 +175,33 @@ class InfoTime(sqltypes.Time):
         return process
 
 colspecs = {
-    sqltypes.DateTime : InfoDateTime,
+    sqltypes.DateTime: InfoDateTime,
     sqltypes.TIMESTAMP: InfoDateTime,
     sqltypes.Time: InfoTime,
 }
 
 
 ischema_names = {
-    0   : sqltypes.CHAR,       # CHAR
-    1   : sqltypes.SMALLINT, # SMALLINT
-    2   : sqltypes.INTEGER,      # INT
-    3   : sqltypes.FLOAT,      # Float
-    3   : sqltypes.Float,      # SmallFloat
-    5   : sqltypes.DECIMAL,      # DECIMAL
-    6   : sqltypes.Integer,      # Serial
-    7   : sqltypes.DATE,         # DATE
-    8   : sqltypes.Numeric,      # MONEY
-    10  : sqltypes.DATETIME,     # DATETIME
-    11  : sqltypes.LargeBinary,       # BYTE
-    12  : sqltypes.TEXT,         # TEXT
-    13  : sqltypes.VARCHAR,       # VARCHAR
-    15  : sqltypes.NCHAR,       # NCHAR
-    16  : sqltypes.NVARCHAR,       # NVARCHAR
-    17  : sqltypes.Integer,      # INT8
-    18  : sqltypes.Integer,      # Serial8
-    43  : sqltypes.String,       # LVARCHAR
-    -1  : sqltypes.BLOB,       # BLOB
-    -1  : sqltypes.CLOB,         # CLOB
+    0: sqltypes.CHAR,           # CHAR
+    1: sqltypes.SMALLINT,       # SMALLINT
+    2: sqltypes.INTEGER,        # INT
+    3: sqltypes.FLOAT,          # Float
+    3: sqltypes.Float,          # SmallFloat
+    5: sqltypes.DECIMAL,        # DECIMAL
+    6: sqltypes.Integer,        # Serial
+    7: sqltypes.DATE,           # DATE
+    8: sqltypes.Numeric,        # MONEY
+    10: sqltypes.DATETIME,      # DATETIME
+    11: sqltypes.LargeBinary,   # BYTE
+    12: sqltypes.TEXT,          # TEXT
+    13: sqltypes.VARCHAR,       # VARCHAR
+    15: sqltypes.NCHAR,         # NCHAR
+    16: sqltypes.NVARCHAR,      # NVARCHAR
+    17: sqltypes.Integer,       # INT8
+    18: sqltypes.Integer,       # Serial8
+    43: sqltypes.String,        # LVARCHAR
+    -1: sqltypes.BLOB,          # BLOB
+    -1: sqltypes.CLOB,          # CLOB
 }
 
 
@@ -215,7 +221,9 @@ class InfoTypeCompiler(compiler.GenericTypeCompiler):
     def visit_boolean(self, type_):
         return "SMALLINT"
 
+
 class InfoSQLCompiler(compiler.SQLCompiler):
+
     def default_from(self):
         return " from systables where tabname = 'systables' "
 
@@ -249,8 +257,9 @@ class InfoSQLCompiler(compiler.SQLCompiler):
         else:
             return compiler.SQLCompiler.visit_function(self, func, **kw)
 
-    def visit_mod(self, binary, **kw):
-        return "MOD(%s, %s)" % (self.process(binary.left), self.process(binary.right))
+    def visit_mod_binary(self, binary, operator, **kw):
+        return "MOD(%s, %s)" % (self.process(binary.left, **kw),
+                                self.process(binary.right, **kw))
 
 
 class InfoDDLCompiler(compiler.DDLCompiler):
@@ -334,6 +343,7 @@ class InfoDDLCompiler(compiler.DDLCompiler):
             text += "CONSTRAINT %s " % self.preparer.format_constraint(constraint)
         return text
 
+
 class InformixIdentifierPreparer(compiler.IdentifierPreparer):
 
     reserved_words = RESERVED_WORDS
@@ -342,7 +352,7 @@ class InformixIdentifierPreparer(compiler.IdentifierPreparer):
 class InformixDialect(default.DefaultDialect):
     name = 'informix'
 
-    max_identifier_length = 128 # adjusts at runtime based on server version
+    max_identifier_length = 128  # adjusts at runtime based on server version
 
     type_compiler = InfoTypeCompiler
     statement_compiler = InfoSQLCompiler
@@ -352,10 +362,6 @@ class InformixDialect(default.DefaultDialect):
     preparer = InformixIdentifierPreparer
     default_paramstyle = 'qmark'
 
-    def __init__(self, has_transactions=True, *args, **kwargs):
-        self.has_transactions = has_transactions
-        default.DefaultDialect.__init__(self, *args, **kwargs)
-
     def initialize(self, connection):
         super(InformixDialect, self).initialize(connection)
 
@@ -365,20 +371,6 @@ class InformixDialect(default.DefaultDialect):
         else:
             self.max_identifier_length = 128
 
-    def do_begin(self, connection):
-        cu = connection.cursor()
-        cu.execute('SET LOCK MODE TO WAIT')
-        if self.has_transactions:
-            cu.execute('SET ISOLATION TO REPEATABLE READ')
-
-    def do_commit(self, connection):
-        if self.has_transactions:
-            connection.commit()
-
-    def do_rollback(self, connection):
-        if self.has_transactions:
-            connection.rollback()
-
     def _get_table_names(self, connection, schema, type, **kw):
         schema = schema or self.default_schema_name
         s = "select tabname, owner from systables where owner=? and tabtype=?"
@@ -414,7 +406,8 @@ class InformixDialect(default.DefaultDialect):
                   and t3.tabid = t2.tabid and t3.colno = t1.colno
                 order by t1.colno""", table_name, schema)
 
-        primary_cols = self.get_primary_keys(connection, table_name, schema, **kw)
+        pk_constraint = self.get_pk_constraint(connection, table_name, schema, **kw)
+        primary_cols = pk_constraint['constrained_columns']
 
         columns = []
         rows = c.fetchall()
@@ -434,14 +427,14 @@ class InformixDialect(default.DefaultDialect):
             if coltype not in (0, 13) and default:
                 default = default.split()[-1]
 
-            if coltype == 6: # Serial, mark as autoincrement
+            if coltype == 6:  # Serial, mark as autoincrement
                 autoincrement = True
 
-            if coltype == 0 or coltype == 13: # char, varchar
+            if coltype == 0 or coltype == 13:  # char, varchar
                 coltype = ischema_names[coltype](collength)
                 if default:
                     default = "'%s'" % default
-            elif coltype == 5: # decimal
+            elif coltype == 5:  # decimal
                 precision, scale = (collength & 0xFF00) >> 8, collength & 0xFF
                 if scale == 255:
                     scale = 0
@@ -483,14 +476,13 @@ class InformixDialect(default.DefaultDialect):
              t8.idxname
              and t7.tabid = t5.ptabid""", table_name, schema_sel)
 
-
         def fkey_rec():
             return {
-                 'name' : None,
-                 'constrained_columns' : [],
-                 'referred_schema' : None,
-                 'referred_table' : None,
-                 'referred_columns' : []
+                 'name': None,
+                 'constrained_columns': [],
+                 'referred_schema': None,
+                 'referred_table': None,
+                 'referred_columns': []
              }
 
         fkeys = util.defaultdict(fkey_rec)
@@ -517,7 +509,7 @@ class InformixDialect(default.DefaultDialect):
         return fkeys.values()
 
     @reflection.cache
-    def get_primary_keys(self, connection, table_name, schema=None, **kw):
+    def get_pk_constraint(self, connection, table_name, schema=None, **kw):
         schema = schema or self.default_schema_name
 
         # Select the column positions from sysindexes for sysconstraints
@@ -532,15 +524,15 @@ class InformixDialect(default.DefaultDialect):
         colpositions = set()
 
         for row in data:
-            colpos = set([getattr(row, 'part%d' % x) for x in range(1,16)])
+            colpos = set([getattr(row, 'part%d' % x) for x in range(1, 16)])
             colpositions |= colpos
 
         if not len(colpositions):
-            return []
+            return {'constrained_columns': [], 'name': None}
 
         # Select the column names using the columnpositions
         # TODO: Maybe cache a bit of those col infos (eg select all colnames for one table)
-        place_holder = ','.join('?'*len(colpositions))
+        place_holder = ','.join('?' * len(colpositions))
         c = connection.execute(
             """select t1.colname
             from syscolumns as t1, systables as t2
@@ -549,7 +541,8 @@ class InformixDialect(default.DefaultDialect):
             table_name, *colpositions
         ).fetchall()
 
-        return reduce(lambda x,y: list(x)+list(y), c, [])
+        cols = reduce(lambda x, y: list(x) + list(y), c, [])
+        return {'constrained_columns': cols, 'name': None}
 
     @reflection.cache
     def get_indexes(self, connection, table_name, schema, **kw):
@@ -562,9 +555,9 @@ class InformixDialect(default.DefaultDialect):
 
         indexes = []
         for row in c.fetchall():
-            colnames = [getattr(row, 'part%d' % x) for x in range(1,16)]
+            colnames = [getattr(row, 'part%d' % x) for x in range(1, 16)]
             colnames = [x for x in colnames if x]
-            place_holder = ','.join('?'*len(colnames))
+            place_holder = ','.join('?' * len(colnames))
             c = connection.execute(
                 """select t1.colname
                 from syscolumns as t1, systables as t2
@@ -572,7 +565,7 @@ class InformixDialect(default.DefaultDialect):
                 t1.colno in (%s)""" % place_holder,
                 table_name, *colnames
             ).fetchall()
-            c = reduce(lambda x,y: list(x)+list(y), c, [])
+            c = reduce(lambda x, y: list(x) + list(y), c, [])
             indexes.append({
                 'name': row.idxname,
                 'unique': row.idxtype.lower() == 'u',
