@@ -1,26 +1,25 @@
 
 from sqlalchemy.ext import serializer
-from sqlalchemy import exc
-import sqlalchemy as sa
-from sqlalchemy.test import testing
-from sqlalchemy import MetaData, Integer, String, ForeignKey, select, \
+from sqlalchemy import testing
+from sqlalchemy import Integer, String, ForeignKey, select, \
     desc, func, util
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 from sqlalchemy.orm import relationship, sessionmaker, scoped_session, \
-    class_mapper, mapper, joinedload, compile_mappers, aliased
-from sqlalchemy.test.testing import eq_
-from test.orm._base import ComparableEntity, MappedTest
+    class_mapper, mapper, joinedload, configure_mappers, aliased
+from sqlalchemy.testing import eq_
 
+from sqlalchemy.testing import fixtures
 
-class User(ComparableEntity):
+class User(fixtures.ComparableEntity):
     pass
 
-class Address(ComparableEntity):
+class Address(fixtures.ComparableEntity):
     pass
 
+users = addresses = Session = None
 
-class SerializeTest(MappedTest):
+class SerializeTest(fixtures.MappedTest):
 
     run_setup_mappers = 'once'
     run_inserts = 'once'
@@ -44,7 +43,7 @@ class SerializeTest(MappedTest):
                : relationship(Address, backref='user',
                order_by=addresses.c.id)})
         mapper(Address, addresses)
-        compile_mappers()
+        configure_mappers()
 
     @classmethod
     def insert_data(cls):
@@ -78,6 +77,7 @@ class SerializeTest(MappedTest):
         assert serializer.loads(serializer.dumps(User.name, -1), None,
                                 None) is User.name
 
+    @testing.requires.python26  # crashes in 2.5
     def test_expression(self):
         expr = \
             select([users]).select_from(users.join(addresses)).limit(5)
@@ -88,19 +88,21 @@ class SerializeTest(MappedTest):
         eq_(re_expr.execute().fetchall(), [(7, u'jack'), (8, u'ed'),
             (8, u'ed'), (8, u'ed'), (9, u'fred')])
 
-    def test_query(self):
-        q = Session.query(User).filter(User.name == 'ed'
-                ).options(joinedload(User.addresses))
-        eq_(q.all(), [User(name='ed', addresses=[Address(id=2),
-            Address(id=3), Address(id=4)])])
-        q2 = serializer.loads(serializer.dumps(q, -1), users.metadata,
-                              Session)
+    def test_query_one(self):
+        q = Session.query(User).\
+                filter(User.name == 'ed').\
+                    options(joinedload(User.addresses))
 
+        q2 = serializer.loads(
+                    serializer.dumps(q, -1),
+                            users.metadata, Session)
         def go():
-            eq_(q2.all(), [User(name='ed', addresses=[Address(id=2),
-                Address(id=3), Address(id=4)])])
+            eq_(q2.all(), [
+                    User(name='ed', addresses=[Address(id=2),
+                    Address(id=3), Address(id=4)])])
 
         self.assert_sql_count(testing.db, go, 1)
+
         eq_(q2.join(User.addresses).filter(Address.email
             == 'ed@bettyboop.com').value(func.count('*')), 1)
         u1 = Session.query(User).get(8)
@@ -111,21 +113,50 @@ class SerializeTest(MappedTest):
         eq_(q2.all(), [Address(email='ed@wood.com'),
             Address(email='ed@lala.com'),
             Address(email='ed@bettyboop.com')])
+
+    @testing.skip_if(lambda: util.pypy, "problems with pypy pickle reported")
+    def test_query_two(self):
         q = \
             Session.query(User).join(User.addresses).\
-                filter(Address.email.like('%fred%'))
+               filter(Address.email.like('%fred%'))
         q2 = serializer.loads(serializer.dumps(q, -1), users.metadata,
                               Session)
         eq_(q2.all(), [User(name='fred')])
-        eq_(list(q2.values(User.id, User.name)), [(9, u'fred')])
+        eq_(list(q2.values(User.id, User.name)), [(9, 'fred')])
 
+    @testing.requires.bulletproof_pickle
+    def test_query_three(self):
+        ua = aliased(User)
+        q = \
+            Session.query(ua).join(ua.addresses).\
+               filter(Address.email.like('%fred%'))
+        q2 = serializer.loads(serializer.dumps(q, -1), users.metadata,
+                              Session)
+        eq_(q2.all(), [User(name='fred')])
+
+        # try to pull out the aliased entity here...
+        ua_2 = q2._entities[0].entity_zero.entity
+        eq_(list(q2.values(ua_2.id, ua_2.name)), [(9, u'fred')])
+
+    @testing.skip_if(lambda: util.pypy, "problems with pypy pickle reported")
+    def test_orm_join(self):
+        from sqlalchemy.orm.util import join
+
+        j = join(User, Address, User.addresses)
+
+        j2 = serializer.loads(serializer.dumps(j, -1), users.metadata)
+        assert j2.left is j.left
+        assert j2.right is j.right
+        assert j2._target_adapter._next
+
+    @testing.requires.python26 # namedtuple workaround not serializable in 2.5
     @testing.exclude('sqlite', '<=', (3, 5, 9),
                      'id comparison failing on the buildbot')
     def test_aliases(self):
         u7, u8, u9, u10 = Session.query(User).order_by(User.id).all()
         ualias = aliased(User)
-        q = Session.query(User, ualias).join((ualias, User.id
-                < ualias.id)).filter(User.id < 9).order_by(User.id,
+        q = Session.query(User, ualias).join(ualias, User.id
+                < ualias.id).filter(User.id < 9).order_by(User.id,
                 ualias.id)
         eq_(list(q.all()), [(u7, u8), (u7, u9), (u7, u10), (u8, u9),
             (u8, u10)])
@@ -134,6 +165,8 @@ class SerializeTest(MappedTest):
         eq_(list(q2.all()), [(u7, u8), (u7, u9), (u7, u10), (u8, u9),
             (u8, u10)])
 
+    @testing.skip_if(lambda: util.pypy, "pickle sometimes has "
+                        "problems here, sometimes not")
     def test_any(self):
         r = User.addresses.any(Address.email == 'x')
         ser = serializer.dumps(r, -1)
